@@ -3,11 +3,14 @@
  *
  *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
@@ -24,6 +27,8 @@
 #include "marshals.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mootype-macros.h"
+#include "mooutils/mooatom.h"
 #include "glade/mooeditprogress-gxml.h"
 #include <string.h>
 #include <stdlib.h>
@@ -33,8 +38,7 @@
 #define KEY_LINE "line"
 
 
-GSList *_moo_edit_instances = NULL;
-
+MooEditList *_moo_edit_instances = NULL;
 
 static GObject *moo_edit_constructor        (GType                  type,
                                              guint                  n_construct_properties,
@@ -107,10 +111,11 @@ enum {
     PROP_EDITOR,
     PROP_ENABLE_BOOKMARKS,
     PROP_HAS_COMMENTS,
+    PROP_LINE_END_TYPE,
     PROP_ENCODING
 };
 
-/* MOO_TYPE_EDIT */
+MOO_DEFINE_BOXED_TYPE_C (MooEditFileInfo, moo_edit_file_info)
 G_DEFINE_TYPE (MooEdit, moo_edit, MOO_TYPE_TEXT_VIEW)
 
 
@@ -147,7 +152,7 @@ moo_edit_class_init (MooEditClass *klass)
                                              "editor",
                                              "editor",
                                              MOO_TYPE_EDITOR,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ENABLE_BOOKMARKS,
@@ -155,7 +160,7 @@ moo_edit_class_init (MooEditClass *klass)
                                              "enable-bookmarks",
                                              "enable-bookmarks",
                                              TRUE,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_HAS_COMMENTS,
@@ -165,18 +170,23 @@ moo_edit_class_init (MooEditClass *klass)
                                              FALSE,
                                              G_PARAM_READABLE));
 
+    g_object_class_install_property (gobject_class, PROP_LINE_END_TYPE,
+        g_param_spec_enum ("line-end-type", "line-end-type", "line-end-type",
+                           MOO_TYPE_LINE_END_TYPE, MOO_LE_NONE,
+                           (GParamFlags) G_PARAM_READWRITE));
+
     g_object_class_install_property (gobject_class,
                                      PROP_ENCODING,
                                      g_param_spec_string ("encoding",
                                              "encoding",
                                              "encoding",
                                              NULL,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     signals[CONFIG_NOTIFY] =
             g_signal_new ("config-notify",
                           G_OBJECT_CLASS_TYPE (klass),
-                          G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED,
+                          (GSignalFlags) (G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED),
                           G_STRUCT_OFFSET (MooEditClass, config_notify),
                           NULL, NULL,
                           _moo_marshal_VOID__UINT_POINTER,
@@ -205,7 +215,7 @@ moo_edit_class_init (MooEditClass *klass)
     signals[COMMENT] =
             _moo_signal_new_cb ("comment",
                                 G_OBJECT_CLASS_TYPE (klass),
-                                G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                (GSignalFlags) (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
                                 G_CALLBACK (moo_edit_comment),
                                 NULL, NULL,
                                 _moo_marshal_VOID__VOID,
@@ -214,7 +224,7 @@ moo_edit_class_init (MooEditClass *klass)
     signals[UNCOMMENT] =
             _moo_signal_new_cb ("uncomment",
                                 G_OBJECT_CLASS_TYPE (klass),
-                                G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                (GSignalFlags) (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
                                 G_CALLBACK (moo_edit_uncomment),
                                 NULL, NULL,
                                 _moo_marshal_VOID__VOID,
@@ -265,7 +275,9 @@ moo_edit_init (MooEdit *edit)
 
     edit->priv->actions = moo_action_collection_new ("MooEdit", "MooEdit");
 
-    indent = moo_indenter_new (edit, NULL);
+    edit->priv->line_end_type = MOO_LE_NONE;
+
+    indent = moo_indenter_new (edit);
     moo_text_view_set_indenter (MOO_TEXT_VIEW (edit), indent);
     g_object_unref (indent);
 }
@@ -286,7 +298,7 @@ moo_edit_constructor (GType                  type,
     edit = MOO_EDIT (object);
 
     _moo_edit_add_class_actions (edit);
-    _moo_edit_instances = g_slist_prepend (_moo_edit_instances, edit);
+    _moo_edit_instances = moo_edit_list_prepend (_moo_edit_instances, edit);
 
     edit->priv->modified_changed_handler_id =
             g_signal_connect (get_buffer (edit),
@@ -294,7 +306,7 @@ moo_edit_constructor (GType                  type,
                               G_CALLBACK (modified_changed_cb),
                               edit);
 
-    _moo_edit_set_filename (edit, NULL, NULL);
+    _moo_edit_set_file (edit, NULL, NULL);
 
     buffer = get_buffer (edit);
     g_signal_connect_swapped (buffer, "line-mark-moved",
@@ -313,6 +325,8 @@ moo_edit_finalize (GObject *object)
 {
     MooEdit *edit = MOO_EDIT (object);
 
+    if (edit->priv->file)
+        g_object_unref (edit->priv->file);
     g_free (edit->priv->filename);
     g_free (edit->priv->display_filename);
     g_free (edit->priv->display_basename);
@@ -328,7 +342,7 @@ moo_edit_dispose (GObject *object)
 {
     MooEdit *edit = MOO_EDIT (object);
 
-    _moo_edit_instances = g_slist_remove (_moo_edit_instances, edit);
+    _moo_edit_instances = moo_edit_list_remove (_moo_edit_instances, edit);
 
     if (edit->config)
     {
@@ -393,6 +407,17 @@ modified_changed_cb (GtkTextBuffer      *buffer,
 }
 
 
+static void
+modify_status (MooEdit       *edit,
+               MooEditStatus  status,
+               gboolean       add)
+{
+    if (add)
+        edit->priv->status = (MooEditStatus) (edit->priv->status | status);
+    else
+        edit->priv->status = (MooEditStatus) (edit->priv->status & ~status);
+}
+
 void
 moo_edit_set_modified (MooEdit            *edit,
                        gboolean            modified)
@@ -416,10 +441,7 @@ moo_edit_set_modified (MooEdit            *edit,
                                   edit->priv->modified_changed_handler_id);
     }
 
-    if (modified)
-        edit->priv->status |= MOO_EDIT_MODIFIED;
-    else
-        edit->priv->status &= ~MOO_EDIT_MODIFIED;
+    modify_status (edit, MOO_EDIT_MODIFIED, modified);
 
     moo_edit_status_changed (edit);
 }
@@ -430,10 +452,7 @@ moo_edit_set_clean (MooEdit            *edit,
                     gboolean            clean)
 {
     g_return_if_fail (MOO_IS_EDIT (edit));
-    if (clean)
-        edit->priv->status |= MOO_EDIT_CLEAN;
-    else
-        edit->priv->status &= ~MOO_EDIT_CLEAN;
+    modify_status (edit, MOO_EDIT_CLEAN, clean);
     moo_edit_status_changed (edit);
 }
 
@@ -454,7 +473,6 @@ moo_edit_status_changed (MooEdit *edit)
 }
 
 
-#if 0
 void
 _moo_edit_set_status (MooEdit        *edit,
                       MooEditStatus   status)
@@ -467,41 +485,6 @@ _moo_edit_set_status (MooEdit        *edit,
         moo_edit_status_changed (edit);
     }
 }
-#endif
-
-
-MooEditFileInfo*
-moo_edit_file_info_new (const char *filename,
-                        const char *encoding)
-{
-    MooEditFileInfo *info = g_new0 (MooEditFileInfo, 1);
-    info->filename = g_strdup (filename);
-    info->encoding = g_strdup (encoding);
-    return info;
-}
-
-
-MooEditFileInfo*
-moo_edit_file_info_copy (const MooEditFileInfo *info)
-{
-    MooEditFileInfo *copy;
-    g_return_val_if_fail (info != NULL, NULL);
-    copy = g_new (MooEditFileInfo, 1);
-    copy->encoding = g_strdup (info->encoding);
-    copy->filename = g_strdup (info->filename);
-    return copy;
-}
-
-void
-moo_edit_file_info_free (MooEditFileInfo *info)
-{
-    if (info)
-    {
-        g_free (info->encoding);
-        g_free (info->filename);
-        g_free (info);
-    }
-}
 
 
 gboolean
@@ -530,7 +513,7 @@ moo_edit_is_untitled (MooEdit *edit)
 MooEditStatus
 moo_edit_get_status (MooEdit *edit)
 {
-    g_return_val_if_fail (MOO_IS_EDIT (edit), 0);
+    g_return_val_if_fail (MOO_IS_EDIT (edit), (MooEditStatus) 0);
     return edit->priv->status;
 }
 
@@ -546,7 +529,7 @@ moo_edit_set_property (GObject        *object,
     switch (prop_id)
     {
         case PROP_EDITOR:
-            edit->priv->editor = g_value_get_object (value);
+            edit->priv->editor = (MooEditor*) g_value_get_object (value);
             break;
 
         case PROP_ENABLE_BOOKMARKS:
@@ -554,7 +537,11 @@ moo_edit_set_property (GObject        *object,
             break;
 
         case PROP_ENCODING:
-            _moo_edit_set_encoding (edit, g_value_get_string (value));
+            moo_edit_set_encoding (edit, g_value_get_string (value));
+            break;
+
+        case PROP_LINE_END_TYPE:
+            moo_edit_set_line_end_type (edit, (MooLineEndType) g_value_get_enum (value));
             break;
 
         default:
@@ -590,6 +577,10 @@ moo_edit_get_property (GObject        *object,
             g_value_set_string (value, edit->priv->encoding);
             break;
 
+        case PROP_LINE_END_TYPE:
+            g_value_set_enum (value, edit->priv->line_end_type);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -638,17 +629,56 @@ moo_edit_focus_out (GtkWidget     *widget,
 }
 
 
-GType
-moo_edit_file_info_get_type (void)
+static MooEditFileInfo *
+moo_edit_file_info_new (GFile      *file,
+                        const char *encoding)
 {
-    static GType type = 0;
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooEditFileInfo",
-                                             (GBoxedCopyFunc) moo_edit_file_info_copy,
-                                             (GBoxedFreeFunc) moo_edit_file_info_free);
-    return type;
+    MooEditFileInfo *info = moo_new (MooEditFileInfo);
+    info->file = file;
+    info->encoding = g_strdup (encoding);
+    return info;
 }
 
+MooEditFileInfo *
+moo_edit_file_info_new_path (const char *path,
+                             const char *encoding)
+{
+    g_return_val_if_fail (path != NULL, NULL);
+    return moo_edit_file_info_new (g_file_new_for_path (path), encoding);
+}
+
+MooEditFileInfo *
+moo_edit_file_info_new_uri (const char *uri,
+                            const char *encoding)
+{
+    g_return_val_if_fail (uri != NULL, NULL);
+    return moo_edit_file_info_new (g_file_new_for_uri (uri), encoding);
+}
+
+MooEditFileInfo *
+moo_edit_file_info_copy (MooEditFileInfo *info)
+{
+    return info ? moo_edit_file_info_new ((GFile*) g_object_ref (info->file), info->encoding) : NULL;
+}
+
+void
+moo_edit_file_info_free (MooEditFileInfo *info)
+{
+    if (info)
+    {
+        g_free (info->encoding);
+        g_object_unref (info->file);
+        moo_free (MooEditFileInfo, info);
+    }
+}
+
+
+GFile *
+_moo_edit_get_file (MooEdit *edit)
+{
+    g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
+    return edit->priv->file ? g_file_dup (edit->priv->file) : NULL;
+}
 
 char *
 moo_edit_get_filename (MooEdit *edit)
@@ -675,11 +705,7 @@ char *
 moo_edit_get_uri (MooEdit *edit)
 {
     g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
-
-    if (edit->priv->filename)
-        return g_filename_to_uri (edit->priv->filename, NULL, NULL);
-    else
-        return NULL;
+    return edit->priv->file ? g_file_get_uri (edit->priv->file) : NULL;
 }
 
 const char *
@@ -690,8 +716,8 @@ moo_edit_get_encoding (MooEdit *edit)
 }
 
 void
-_moo_edit_set_encoding (MooEdit    *edit,
-                        const char *encoding)
+moo_edit_set_encoding (MooEdit    *edit,
+                       const char *encoding)
 {
     g_return_if_fail (MOO_IS_EDIT (edit));
     g_return_if_fail (encoding != NULL);
@@ -1063,7 +1089,7 @@ moo_edit_apply_config (MooEdit *edit)
                          "show-line-numbers", &line_numbers,
                          "tab-width", &tab_width,
                          "word-chars", &word_chars,
-                         NULL);
+                         (char*) 0);
 
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (edit), wrap_mode);
     moo_text_view_set_show_line_numbers (MOO_TEXT_VIEW (edit), line_numbers);
@@ -1118,7 +1144,7 @@ _moo_edit_update_config_from_global (MooEdit *edit)
 void
 _moo_edit_update_lang_config (void)
 {
-    GSList *l;
+    MooEditList *l;
 
     for (l = _moo_edit_instances; l != NULL; l = l->next)
     {
@@ -1154,7 +1180,7 @@ moo_edit_filename_changed (MooEdit    *edit,
     }
 
     moo_edit_config_set (edit->config, MOO_EDIT_CONFIG_SOURCE_FILENAME,
-                         "lang", lang_id, "indent", NULL, NULL);
+                         "lang", lang_id, "indent", (void*) NULL, (char*) NULL);
 
     if (filter_config)
         moo_edit_config_parse (edit->config, filter_config,
@@ -1245,7 +1271,7 @@ find_uri_atom (GdkDragContext *context)
     GList *targets;
     GdkAtom atom;
 
-    atom = gdk_atom_intern ("text/uri-list", FALSE);
+    atom = moo_atom_uri_list ();
     targets = context->targets;
 
     while (targets)
@@ -1455,14 +1481,14 @@ block_uncomment (GtkTextBuffer *buffer,
     }
 
     limit = *end;
-    found = moo_text_search_forward (start, comment_start, 0,
+    found = moo_text_search_forward (start, comment_start, (MooTextSearchFlags) 0,
                                      &start1, &start2,
                                      &limit);
 
     if (!found)
     {
         gtk_text_iter_set_line_offset (&limit, 0);
-        found = gtk_text_iter_backward_search (start, comment_start, 0,
+        found = gtk_text_iter_backward_search (start, comment_start, (GtkTextSearchFlags) 0,
                                                &start1, &start2,
                                                &limit);
     }
@@ -1471,14 +1497,14 @@ block_uncomment (GtkTextBuffer *buffer,
         return;
 
     limit = start2;
-    found = gtk_text_iter_backward_search (end, comment_end, 0,
+    found = gtk_text_iter_backward_search (end, comment_end, (GtkTextSearchFlags) 0,
                                            &end1, &end2, &limit);
 
     if (!found)
     {
         limit = *end;
         iter_to_line_end (&limit);
-        found = moo_text_search_forward (end, comment_end, 0,
+        found = moo_text_search_forward (end, comment_end, (MooTextSearchFlags) 0,
                                          &end1, &end2, &limit);
     }
 
@@ -1686,7 +1712,7 @@ void
 _moo_edit_do_popup (MooEdit        *edit,
                     GdkEventButton *event)
 {
-    MooUIXML *xml;
+    MooUiXml *xml;
     MooEditWindow *window;
     GtkMenu *menu;
 
@@ -1694,8 +1720,8 @@ _moo_edit_do_popup (MooEdit        *edit,
     xml = moo_editor_get_doc_ui_xml (edit->priv->editor);
     g_return_if_fail (xml != NULL);
 
-    menu = moo_ui_xml_create_widget (xml, MOO_UI_MENU, "Editor/Popup", edit->priv->actions,
-                                     window ? MOO_WINDOW(window)->accel_group : NULL);
+    menu = (GtkMenu*) moo_ui_xml_create_widget (xml, MOO_UI_MENU, "Editor/Popup", edit->priv->actions,
+                                                window ? MOO_WINDOW(window)->accel_group : NULL);
     g_return_if_fail (menu != NULL);
     MOO_OBJECT_REF_SINK (menu);
 
@@ -1894,6 +1920,7 @@ _moo_edit_set_state (MooEdit        *edit,
 
 static struct {
     char *working_dir;
+    char *encodings_dir;
 } test_data;
 
 #ifdef __WIN32__
@@ -1992,11 +2019,105 @@ test_basic (void)
     g_free (filename);
 }
 
+#define TEST_ASSERT_SAME_FILE_CONTENT(filename1, filename2)             \
+{                                                                       \
+    char *contents1__ = NULL;                                           \
+    char *contents2__ = NULL;                                           \
+    g_file_get_contents (filename1, &contents1__, NULL, NULL);          \
+    g_file_get_contents (filename2, &contents2__, NULL, NULL);          \
+    if (!contents1__)                                                   \
+        moo_test_assert_msg (FALSE, __FILE__, __LINE__,                 \
+                             "could not open file %s",                  \
+                             filename1);                                \
+    if (!contents2__)                                                   \
+        moo_test_assert_msg (FALSE, __FILE__, __LINE__,                 \
+                             "could not open file %s",                  \
+                             filename2);                                \
+    if (contents1__ && contents2__)                                     \
+    {                                                                   \
+        gboolean equal = strcmp (contents1__, contents2__) == 0;        \
+        TEST_ASSERT_MSG (equal, "contents of %s and %s differ",         \
+                         filename1, filename2);                         \
+    }                                                                   \
+    g_free (contents2__);                                               \
+    g_free (contents1__);                                               \
+}
+
+static void
+test_encodings_1 (const char *name,
+                  const char *working_dir)
+{
+    char *filename;
+    char *filename2;
+    char *encoding;
+    const char *dot;
+    MooEditor *editor;
+    MooEdit *doc;
+
+    if ((dot = strchr (name, '.')))
+        encoding = g_strndup (name, dot - name);
+    else
+        encoding = g_strdup (name);
+
+    filename = g_build_filename (test_data.encodings_dir, name, (char*)0);
+    filename2 = g_build_filename (working_dir, name, (char*)0);
+
+    editor = moo_editor_instance ();
+    doc = moo_editor_open_file (editor, NULL, NULL, filename, encoding);
+    TEST_ASSERT (doc != NULL);
+
+    if (doc)
+    {
+        TEST_ASSERT (moo_edit_save_as (doc, filename2, NULL, NULL));
+        TEST_ASSERT_SAME_FILE_CONTENT (filename2, filename);
+        TEST_ASSERT (moo_edit_close (doc, TRUE));
+    }
+
+    g_free (encoding);
+    g_free (filename2);
+    g_free (filename);
+}
+
+static void
+test_encodings (void)
+{
+    GDir *dir;
+    const char *name;
+    char *working_dir;
+
+    dir = g_dir_open (test_data.encodings_dir, 0, NULL);
+
+    if (!dir)
+    {
+        g_critical ("could not open encodings dir");
+        TEST_ASSERT (FALSE);
+        return;
+    }
+
+    working_dir = g_build_filename (test_data.working_dir, "encodings", (char*)0);
+    _moo_mkdir_with_parents (working_dir);
+
+    while ((name = g_dir_read_name (dir)))
+        test_encodings_1 (name, working_dir);
+
+    g_free (working_dir);
+    g_dir_close (dir);
+}
+
 static gboolean
 test_suite_init (G_GNUC_UNUSED gpointer data)
 {
+    /* try to avoid accidental deletion of good stuff */
+    if (strcmp (g_get_prgname(), "run-tests") != 0)
+    {
+        g_critical ("fix the program name!");
+        return FALSE;
+    }
+
     test_data.working_dir = g_build_filename (moo_test_get_working_dir (),
-                                              "editor-work", NULL);
+                                              "editor-work", (char*)0);
+    test_data.encodings_dir = g_build_filename (moo_test_get_data_dir (),
+                                                "encodings", (char*)0);
 
     if (_moo_mkdir_with_parents (test_data.working_dir) != 0)
     {
@@ -2015,19 +2136,42 @@ static void
 test_suite_cleanup (G_GNUC_UNUSED gpointer data)
 {
     GError *error = NULL;
+    char *recent_file;
+    char *cache_dir;
+    MooEditor *editor;
 
-    if (!_moo_remove_dir (test_data.working_dir, TRUE, &error))
-    {
-        g_critical ("could not remove directory '%s': %s",
-                    test_data.working_dir, error->message);
-        g_error_free (error);
-    }
+//     if (!_moo_remove_dir (test_data.working_dir, TRUE, &error))
+//     {
+//         g_critical ("could not remove directory '%s': %s",
+//                     test_data.working_dir, error->message);
+//         g_error_free (error);
+//         error = NULL;
+//     }
 
     g_free (test_data.working_dir);
+    g_free (test_data.encodings_dir);
     test_data.working_dir = NULL;
+    test_data.encodings_dir = NULL;
 
-    moo_editor_close_all (moo_editor_instance (), FALSE, FALSE);
+    editor = moo_editor_instance ();
+    moo_editor_close_all (editor, FALSE, FALSE);
+    recent_file = _md_history_mgr_get_filename (_moo_editor_get_history_mgr (editor));
     g_object_unref (moo_editor_instance ());
+
+    if (!g_file_test (recent_file, G_FILE_TEST_EXISTS))
+        g_critical ("recent file %s does not exist", recent_file);
+
+    cache_dir = moo_get_user_cache_dir ();
+    if (!_moo_remove_dir (cache_dir, TRUE, &error))
+    {
+        g_critical ("could not remove directory '%s': %s",
+                    cache_dir, error->message);
+        g_error_free (error);
+        error = NULL;
+    }
+
+    g_free (cache_dir);
+    g_free (recent_file);
 }
 
 void
@@ -2038,6 +2182,7 @@ moo_test_editor (void)
                                               test_suite_cleanup,
                                               NULL);
     moo_test_suite_add_test (suite, "basic", (MooTestFunc) test_basic, NULL);
+    moo_test_suite_add_test (suite, "encodings", (MooTestFunc) test_encodings, NULL);
 }
 
 #endif
