@@ -1,13 +1,16 @@
 /*
  *   moofile.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -21,13 +24,14 @@
 #define MOO_FILE_VIEW_COMPILATION
 #include "moofileview/moofilesystem.h"
 #include "moofileview/moofile-private.h"
-#include "moofileview/moofileicons.h"
+#include "mooutils/moofileicon.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mootype-macros.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/moocompat.h"
 #include "marshals.h"
-#include "mooutils/xdgmime/xdgmime.h"
+#include "mooutils/moo-mime.h"
 #include <glib/gstdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -46,8 +50,6 @@
 
 MOO_DEBUG_INIT(file, FALSE)
 
-#define MIME_TYPE_UNKNOWN xdg_mime_type_unknown
-
 #define NORMAL_PRIORITY         G_PRIORITY_DEFAULT_IDLE
 #define NORMAL_TIMEOUT          0.04
 #define BACKGROUND_PRIORITY     G_PRIORITY_LOW
@@ -59,30 +61,24 @@ G_STMT_START {              \
     g_timer_stop (timer);   \
 } G_STMT_END
 
-static MooIconType   _get_folder_icon           (const char     *path);
-static MooIconEmblem _get_icon_flags            (const MooFile  *file);
+static MooIconType      get_folder_icon (const char     *path);
+static MooIconEmblem    get_icon_flags  (const MooFile  *file);
 
-static GdkPixbuf    *render_icon                (const MooFile  *file,
-                                                 GtkWidget      *widget,
-                                                 GtkIconSize     size);
-static GdkPixbuf    *render_icon_for_path       (const char     *path,
-                                                 GtkWidget      *widget,
-                                                 GtkIconSize     size);
+MOO_DEFINE_BOXED_TYPE_R (MooFile, _moo_file)
 
 #define MAKE_PATH(dirname,file) g_build_filename (dirname, file->name, NULL)
 
-
 void
 _moo_file_find_mime_type (MooFile    *file,
                           const char *path)
 {
-    file->mime_type = xdg_mime_get_mime_type_for_file (path, file->statbuf);
+    file->mime_type = moo_get_mime_type_for_file (path, file->statbuf);
 
     if (!file->mime_type || !file->mime_type[0])
     {
         /* this should not happen */
         _moo_message ("%s: oops, %s", G_STRLOC, file->display_name);
-        file->mime_type = MIME_TYPE_UNKNOWN;
+        file->mime_type = MOO_MIME_TYPE_UNKNOWN;
     }
 
     file->flags |= MOO_FILE_HAS_MIME_TYPE;
@@ -347,7 +343,7 @@ _moo_file_stat (MooFile    *file,
     {
         if (errno == ENOENT)
         {
-            MOO_DEBUG({
+            MOO_DEBUG_CODE({
                 gchar *display_name = g_filename_display_name (fullname);
                 _moo_message ("%s: file '%s' does not exist",
                               G_STRLOC, display_name);
@@ -357,7 +353,7 @@ _moo_file_stat (MooFile    *file,
         }
         else
         {
-            MOO_DEBUG({
+            MOO_DEBUG_CODE({
                 int save_errno = errno;
                 gchar *display_name = g_filename_display_name (fullname);
                 _moo_message ("%s: error getting information for '%s': %s",
@@ -384,7 +380,7 @@ _moo_file_stat (MooFile    *file,
             {
                 if (errno == ENOENT)
                 {
-                    MOO_DEBUG({
+                    MOO_DEBUG_CODE({
                         gchar *display_name = g_filename_display_name (fullname);
                         _moo_message ("%s: file '%s' is a broken link",
                                       G_STRLOC, display_name);
@@ -394,7 +390,7 @@ _moo_file_stat (MooFile    *file,
                 }
                 else
                 {
-                    MOO_DEBUG({
+                    MOO_DEBUG_CODE({
                         int save_errno = errno;
                         gchar *display_name = g_filename_display_name (fullname);
                         _moo_message ("%s: error getting information for '%s': %s",
@@ -411,7 +407,7 @@ _moo_file_stat (MooFile    *file,
 
             if (len == -1)
             {
-                MOO_DEBUG({
+                MOO_DEBUG_CODE({
                     int save_errno = errno;
                     gchar *display_name = g_filename_display_name (fullname);
                     _moo_message ("%s: error getting link target for '%s': %s",
@@ -526,19 +522,15 @@ _moo_file_get_icon (const MooFile  *file,
                     GtkWidget      *widget,
                     GtkIconSize     size)
 {
+    MooFileIcon icon = {0};
+
     g_return_val_if_fail (file != NULL, NULL);
     g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
-    return render_icon (file, widget, size);
-}
 
-
-GdkPixbuf *
-_moo_get_icon_for_path (const char     *path,
-                        GtkWidget      *widget,
-                        GtkIconSize     size)
-{
-    g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
-    return render_icon_for_path (path, widget, size);
+    icon.mime_type = file->mime_type;
+    icon.type = file->icon;
+    icon.emblem = get_icon_flags (file);
+    return moo_file_icon_get_pixbuf (&icon, widget, size);
 }
 
 
@@ -580,97 +572,6 @@ _moo_file_get_stat (const MooFile *file)
 #endif
 
 
-GType
-_moo_file_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooFile",
-                                             (GBoxedCopyFunc) _moo_file_ref,
-                                             (GBoxedFreeFunc) _moo_file_unref);
-
-    return type;
-}
-
-
-GType
-_moo_file_flags_get_type (void)
-{
-    static GType type = 0;
-
-    static const GFlagsValue values[] = {
-        { MOO_FILE_HAS_MIME_TYPE, (char*)"MOO_FILE_HAS_MIME_TYPE", (char*)"has-mime-type" },
-        { MOO_FILE_HAS_ICON, (char*)"MOO_FILE_HAS_ICON", (char*)"has-icon" },
-        { MOO_FILE_HAS_STAT, (char*)"MOO_FILE_HAS_STAT", (char*)"has-stat" },
-        { MOO_FILE_ALL_FLAGS, (char*)"MOO_FILE_ALL_FLAGS", (char*)"all-flags" },
-        { 0, NULL, NULL }
-    };
-
-    if (G_UNLIKELY (!type))
-        type = g_flags_register_static ("MooFileFlags", values);
-
-    return type;
-}
-
-
-GType
-_moo_file_info_get_type (void)
-{
-    static GType type = 0;
-
-    static const GFlagsValue values[] = {
-        { MOO_FILE_INFO_EXISTS, (char*)"MOO_FILE_INFO_EXISTS", (char*)"exists" },
-        { MOO_FILE_INFO_IS_DIR, (char*)"MOO_FILE_INFO_IS_DIR", (char*)"is-folder" },
-        { MOO_FILE_INFO_IS_HIDDEN, (char*)"MOO_FILE_INFO_IS_HIDDEN", (char*)"is-hidden" },
-        { MOO_FILE_INFO_IS_LINK, (char*)"MOO_FILE_INFO_IS_LINK", (char*)"is-link" },
-        { 0, NULL, NULL }
-    };
-
-    if (G_UNLIKELY (!type))
-        type = g_flags_register_static ("MooFileInfo", values);
-
-    return type;
-}
-
-
-static GdkPixbuf *
-render_icon (const MooFile  *file,
-             GtkWidget      *widget,
-             GtkIconSize     size)
-{
-    GdkPixbuf *pixbuf = _moo_get_icon (widget, file->icon, file->mime_type,
-                                       _get_icon_flags (file), size);
-    g_assert (pixbuf != NULL);
-    return pixbuf;
-}
-
-
-static GdkPixbuf *
-render_icon_for_path (const char     *path,
-                      GtkWidget      *widget,
-                      GtkIconSize     size)
-{
-    const char *mime_type = MIME_TYPE_UNKNOWN;
-
-#ifndef __WIN32__
-    if (path)
-    {
-        mime_type = xdg_mime_get_mime_type_for_file (path, NULL);
-
-        if (!mime_type || !mime_type[0])
-            mime_type = MIME_TYPE_UNKNOWN;
-    }
-#else
-#ifdef __GNUC__
-#warning "Implement render_icon_for_path()"
-#endif
-#endif
-
-    return _moo_get_icon (widget, MOO_ICON_MIME, mime_type, 0, size);
-}
-
-
 guint8
 _moo_file_get_icon_type (MooFile    *file,
                          const char *dirname)
@@ -684,7 +585,7 @@ _moo_file_get_icon_type (MooFile    *file,
     if (MOO_FILE_IS_DIR (file))
     {
         char *path = MAKE_PATH (dirname, file);
-        MooIconType icon = _get_folder_icon (path);
+        MooIconType icon = get_folder_icon (path);
         g_free (path);
         return icon;
     }
@@ -716,7 +617,7 @@ _moo_file_icon_blank (void)
 
 
 static MooIconType
-_get_folder_icon (const char *path)
+get_folder_icon (const char *path)
 {
     static const char *home_path = NULL;
     static char *desktop_path = NULL;
@@ -750,7 +651,7 @@ _get_folder_icon (const char *path)
 
 
 static MooIconEmblem
-_get_icon_flags (const MooFile *file)
+get_icon_flags (const MooFile *file)
 {
     return
 #if 0
