@@ -1,13 +1,16 @@
 /*
  *   mooapp/mooappinput.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -43,7 +46,7 @@
 #include "mooutils-thread.h"
 #include "mooutils-debug.h"
 
-MOO_DEBUG_INIT(input, TRUE)
+MOO_DEBUG_INIT(input, FALSE)
 
 #define MAX_BUFFER_SIZE 4096
 #define IPC_MAGIC_CHAR 'I'
@@ -651,7 +654,8 @@ read_input (G_GNUC_UNUSED GIOChannel *source,
     return TRUE;
 
 remove:
-    conn->ch->connections = g_slist_remove (conn->ch->connections, conn);
+    if (conn->ch)
+        conn->ch->connections = g_slist_remove (conn->ch->connections, conn);
     connection_free (conn);
     return FALSE;
 }
@@ -907,7 +911,7 @@ input_channel_get_path (InputChannel *ch)
     return g_strdup (ch->pipe_name);
 }
 
-static const char *
+G_GNUC_UNUSED static const char *
 input_channel_get_name (InputChannel *ch)
 {
     g_return_val_if_fail (ch != NULL, NULL);
@@ -1027,11 +1031,7 @@ event_callback (GList        *events,
         if (c != 0)
             g_string_append_c (ch->buffer, c);
         else
-        {
-            gdk_threads_enter ();
             commit (&ch->buffer);
-            gdk_threads_leave ();
-        }
 
         events = events->next;
     }
@@ -1180,7 +1180,7 @@ _moo_app_input_broadcast (const char *header,
                           const char *data,
                           gssize      len)
 {
-#warning "Implement me"
+    MOO_IMPLEMENT_ME
 
     g_return_if_fail (header != NULL);
     g_return_if_fail (data != NULL);
