@@ -1,13 +1,16 @@
 /*
  *   moofilewatch.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,6 +44,7 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-mem.h"
 #include "mooutils/moofilewatch.h"
+#include "mooutils/mootype-macros.h"
 #include "marshals.h"
 #include "mooutils/mooutils-thread.h"
 
@@ -99,9 +103,6 @@ struct _MooFileWatch {
     guint alive : 1;
 };
 
-
-#define MOO_FILE_WATCH_ERROR (moo_file_watch_error_quark ())
-
 typedef enum
 {
     MOO_FILE_WATCH_ERROR_CLOSED,
@@ -115,8 +116,10 @@ typedef enum
     MOO_FILE_WATCH_ERROR_ACCESS_DENIED
 } MooFileWatchError;
 
-static GQuark moo_file_watch_error_quark (void);
+MOO_DEFINE_BOXED_TYPE_R (MooFileWatch, moo_file_watch)
 
+#define MOO_FILE_WATCH_ERROR (moo_file_watch_error_quark ())
+MOO_DEFINE_QUARK_STATIC (moo-file-watch-error, moo_file_watch_error_quark)
 
 #ifdef MOO_USE_FAM
 static gboolean watch_fam_start             (MooFileWatch   *watch,
@@ -262,30 +265,6 @@ moo_file_watch_new (GError **error)
 }
 
 
-GType
-moo_file_watch_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooFileWatch",
-                                             (GBoxedCopyFunc) moo_file_watch_ref,
-                                             (GBoxedFreeFunc) moo_file_watch_unref);
-
-    return type;
-}
-
-
-static GQuark
-moo_file_watch_error_quark (void)
-{
-    static GQuark quark = 0;
-    if (G_UNLIKELY (!quark))
-        quark = g_quark_from_static_string ("moo-file-watch-error");
-    return quark;
-}
-
-
 static MooFileEvent *
 moo_file_event_new (const char      *filename,
                     guint            monitor_id,
@@ -329,18 +308,7 @@ moo_file_event_free (MooFileEvent *event)
     }
 }
 
-GType
-moo_file_event_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooFileEvent",
-                                             (GBoxedCopyFunc) moo_file_event_copy,
-                                             (GBoxedFreeFunc) moo_file_event_free);
-
-    return type;
-}
+MOO_DEFINE_BOXED_TYPE_C (MooFileEvent, moo_file_event)
 
 
 gboolean
@@ -1073,7 +1041,7 @@ fam_thread_add_path (FAMThread  *thr,
 {
     gunichar2 *win_path;
 
-    if (thr->n_events == MAXIMUM_WAIT_OBJECTS)
+    if (thr->n_events >= MAXIMUM_WAIT_OBJECTS)
     {
         g_critical ("%s: too many folders watched", G_STRLOC);
         return;
@@ -1407,7 +1375,7 @@ fam_win32_init (void)
         fam->running = FALSE;
 
         fam->thread_data.n_events = 1;
-        fam->thread_data.events[0] = CreateEvent (NULL, TRUE, FALSE, NULL);;
+        fam->thread_data.events[0] = CreateEvent (NULL, TRUE, FALSE, NULL);
 
         if (!fam->thread_data.events[0])
         {
