@@ -3,11 +3,14 @@
  *
  *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -15,13 +18,15 @@
 #endif
 
 #include "mooutils/mooutils-fs.h"
-#include "mooutils/mooutils-file.h"
+// #include "mooutils/mooutils-file.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/mooutils-mem.h"
 #include "mooutils/mootype-macros.h"
 #include "mooutils/moocompat.h"
+#include "mooutils/mooi18n.h"
 
 #include <stdio.h>
+#include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <errno.h>
@@ -33,6 +38,8 @@
 #include <windows.h>
 #include <io.h>
 #include <direct.h>
+#include <shellapi.h>
+#include <gdk/gdkwin32.h>
 #endif
 
 #ifndef S_IRWXU
@@ -48,6 +55,7 @@
 
 #define BROKEN_NAME "<" "????" ">"
 
+MOO_DEFINE_QUARK (moo-file-error-quark, _moo_file_error_quark)
 
 /* XXX fix this */
 gboolean
@@ -89,6 +97,111 @@ _moo_save_file_utf8 (const char *name,
 }
 
 
+#ifdef __WIN32__
+
+static wchar_t *
+filename_to_double_null_terminated_string (const char *filename)
+{
+    long len;
+    gunichar2 *ret;
+
+    ret = g_utf8_to_utf16 (filename, -1, NULL, &len, NULL);
+
+    if (!ret)
+        return NULL;
+
+    ret = g_renew (gunichar2, ret, len + 2);
+    ret[len + 1] = 0;
+
+    return (wchar_t*) ret;
+}
+
+static wchar_t *
+filename_list_to_double_null_terminated_string (GList *filenames)
+{
+    GArray *ret;
+    gunichar2 wzero = 0;
+
+    ret = g_array_new (FALSE, FALSE, sizeof (gunichar2));
+
+    while (filenames)
+    {
+        guint old_len;
+        long len;
+        gunichar2 *wstr;
+
+        wstr = g_utf8_to_utf16 (filenames->data, -1, NULL, &len, NULL);
+        filenames = filenames->next;
+
+        if (!wstr)
+        {
+            g_array_free (ret, TRUE);
+            return NULL;
+        }
+
+        old_len = ret->len;
+        g_array_set_size (ret, old_len + len + 1);
+        memcpy (ret->data + old_len * sizeof (gunichar2), wstr, (len + 1) * sizeof (gunichar2));
+
+        g_free (wstr);
+    }
+
+    g_array_append_val (ret, wzero);
+    return (wchar_t*) g_array_free (ret, FALSE);
+}
+
+static gboolean
+move_or_copy_files_ui (GList      *filenames,
+                       const char *destdir,
+                       gboolean    copy,
+                       GtkWidget  *parent)
+{
+    SHFILEOPSTRUCT shop = {0};
+    gboolean ret;
+    wchar_t *from, *to;
+
+    from = filename_list_to_double_null_terminated_string (filenames);
+    to = filename_to_double_null_terminated_string (destdir);
+
+    if (!from || !to)
+    {
+        g_free (from);
+        g_free (to);
+        return FALSE;
+    }
+
+    shop.hwnd = parent && parent->window ? GDK_WINDOW_HWND (parent->window) : NULL;
+    shop.wFunc = copy ? FO_COPY : FO_MOVE;
+    shop.pFrom = from;
+    shop.pTo = to;
+    shop.fFlags = FOF_ALLOWUNDO;
+
+    ret = SHFileOperation(&shop) == 0 && !shop.fAnyOperationsAborted;
+
+    g_free (from);
+    g_free (to);
+    return ret;
+}
+
+gboolean
+_moo_copy_files_ui (GList      *filenames,
+                    const char *destdir,
+                    GtkWidget  *parent)
+{
+    return move_or_copy_files_ui (filenames, destdir, TRUE, parent);
+}
+
+gboolean
+_moo_move_files_ui (GList      *filenames,
+                    const char *destdir,
+                    GtkWidget  *parent)
+{
+    return move_or_copy_files_ui (filenames, destdir, FALSE, parent);
+}
+
+#endif /* __WIN32__ */
+
+
 #ifndef __WIN32__
 static gboolean
 rm_fr (const char *path,
@@ -109,7 +222,9 @@ rm_fr (const char *path,
                        NULL, NULL, NULL, &child_err, &status, &error_here))
     {
         g_set_error (error, MOO_FILE_ERROR, MOO_FILE_ERROR_FAILED,
-                     "Could not run 'rm' command: %s",
+                     /* This is error message in file selector when rm
+                        fails for some strange reason */
+                     _("Could not run 'rm' command: %s"),
                      error_here->message);
         g_error_free (error_here);
         g_strfreev (argv);
@@ -120,10 +235,16 @@ rm_fr (const char *path,
 
     if (!WIFEXITED (status) || WEXITSTATUS (status))
     {
+        if (child_err && strlen (child_err) > 5000)
+            strcpy (child_err + 4997, "...");
+
         g_set_error (error, MOO_FILE_ERROR,
                      MOO_FILE_ERROR_FAILED,
-                     "'rm' command failed: %s",
+                     /* This is error message in file selector when rm
+                        fails to delete a file or folder */
+                     _("'rm' command failed: %s"),
                      child_err ? child_err : "");
+
         g_free (child_err);
         return FALSE;
     }
@@ -171,7 +292,7 @@ rm_r (const char *path,
                 success = FALSE;
                 g_set_error (error, MOO_FILE_ERROR,
                              _moo_file_error_from_errno (err),
-                             "Could not remove '%s': %s", file_path,
+                             _("Could not remove %s: %s"), file_path,
                              g_strerror (err));
             }
         }
@@ -191,7 +312,7 @@ rm_r (const char *path,
             success = FALSE;
             g_set_error (error, MOO_FILE_ERROR,
                          _moo_file_error_from_errno (err),
-                         "Could not remove '%s': %s", path,
+                         _("Could not remove %s: %s"), path,
                          g_strerror (err));
         }
     }
@@ -218,7 +339,7 @@ _moo_remove_dir (const char *path,
             char *path_utf8 = g_filename_display_name (path);
             g_set_error (error, MOO_FILE_ERROR,
                          _moo_file_error_from_errno (err),
-                         "Could not remove '%s': %s",
+                         _("Could not remove %s: %s"),
                          path_utf8, g_strerror (err));
             g_free (path_utf8);
             return FALSE;
@@ -263,7 +384,7 @@ _moo_create_dir (const char *path,
         g_set_error (error,
                      MOO_FILE_ERROR,
                      _moo_file_error_from_errno (err_code),
-                     "Could not create directory '%s': %s",
+                     _("Could not create folder %s: %s"),
                      utf8_path, g_strerror (err_code));
 
         g_free (utf8_path);
@@ -282,7 +403,7 @@ _moo_create_dir (const char *path,
             g_set_error (error,
                          MOO_FILE_ERROR,
                          _moo_file_error_from_errno (err_code),
-                         "Could not create directory '%s': %s",
+                         _("Could not create folder %s: %s"),
                          utf8_path, g_strerror (err_code));
 
             g_free (utf8_path);
@@ -298,7 +419,7 @@ _moo_create_dir (const char *path,
     utf8_path = g_filename_display_name (path);
     g_set_error (error, MOO_FILE_ERROR,
                  MOO_FILE_ERROR_ALREADY_EXISTS,
-                 "Could not create directory '%s': %s",
+                 _("Could not create folder %s: %s"),
                  utf8_path, g_strerror (EEXIST));
     g_free (utf8_path);
 
@@ -327,7 +448,7 @@ _moo_rename_file (const char *path,
         g_set_error (error,
                      MOO_FILE_ERROR,
                      _moo_file_error_from_errno (err_code),
-                     "Could not rename file '%s' to '%s': %s",
+                     _("Could not rename file %s to %s: %s"),
                      utf8_path, utf8_new_path, g_strerror (err_code));
 
         g_free (utf8_path);
@@ -339,18 +460,6 @@ _moo_rename_file (const char *path,
 }
 
 
-GQuark
-_moo_file_error_quark (void)
-{
-    static GQuark quark = 0;
-
-    if (quark == 0)
-        quark = g_quark_from_static_string ("moo-file-error-quark");
-
-    return quark;
-}
-
-
 MooFileError
 _moo_file_error_from_errno (int code)
 {
@@ -1412,599 +1521,3 @@ _moo_glob_match_simple (const char *pattern,
     _moo_glob_free (gl);
     return result;
 }
-
-
-/************************************************************************/
-/* MooFileWriter
- */
-
-typedef struct MooFileWriterClass MooFileWriterClass;
-
-struct MooFileWriter {
-    GObject base;
-};
-
-struct MooFileWriterClass {
-    GObjectClass base_class;
-
-    gboolean (*meth_write)  (MooFileWriter *writer,
-                             const char    *data,
-                             gsize          len);
-    gboolean (*meth_printf) (MooFileWriter  *writer,
-                             const char     *fmt,
-                             va_list         args) G_GNUC_PRINTF (2, 0);
-    gboolean (*meth_close)  (MooFileWriter  *writer,
-                             GError        **error);
-};
-
-#define MOO_TYPE_FILE_WRITER            (moo_file_writer_get_type ())
-#define MOO_FILE_WRITER(object)         (G_TYPE_CHECK_INSTANCE_CAST ((object), MOO_TYPE_FILE_WRITER, MooFileWriter))
-#define MOO_FILE_WRITER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), MOO_TYPE_FILE_WRITER, MooFileWriterClass))
-#define MOO_IS_FILE_WRITER(object)      (G_TYPE_CHECK_INSTANCE_TYPE ((object), MOO_TYPE_FILE_WRITER))
-#define MOO_IS_FILE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MOO_TYPE_FILE_WRITER))
-#define MOO_FILE_WRITER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_FILE_WRITER, MooFileWriterClass))
-
-MOO_DEFINE_TYPE_STATIC (MooFileWriter, moo_file_writer, G_TYPE_OBJECT)
-
-static void
-moo_file_writer_init (G_GNUC_UNUSED MooFileWriter *writer)
-{
-}
-
-static void
-moo_file_writer_class_init (G_GNUC_UNUSED MooFileWriterClass *klass)
-{
-}
-
-gboolean
-moo_file_writer_write (MooFileWriter *writer,
-                       const char    *data,
-                       gssize         len)
-{
-    g_return_val_if_fail (MOO_IS_FILE_WRITER (writer), FALSE);
-    g_return_val_if_fail (data != NULL, FALSE);
-
-    if (len < 0)
-        len = strlen (data);
-    if (!len)
-        return TRUE;
-
-    return MOO_FILE_WRITER_GET_CLASS (writer)->meth_write (writer, data, len);
-}
-
-gboolean
-moo_file_writer_printf (MooFileWriter  *writer,
-                        const char     *fmt,
-                        ...)
-{
-    va_list args;
-    gboolean ret;
-
-    g_return_val_if_fail (MOO_IS_FILE_WRITER (writer), FALSE);
-    g_return_val_if_fail (fmt != NULL, FALSE);
-
-    va_start (args, fmt);
-    ret = MOO_FILE_WRITER_GET_CLASS (writer)->meth_printf (writer, fmt, args);
-    va_end (args);
-
-    return ret;
-}
-
-gboolean
-moo_file_writer_close (MooFileWriter  *writer,
-                       GError        **error)
-{
-    gboolean ret;
-
-    g_return_val_if_fail (MOO_IS_FILE_WRITER (writer), FALSE);
-    g_return_val_if_fail (!error || !*error, FALSE);
-
-    ret = MOO_FILE_WRITER_GET_CLASS (writer)->meth_close (writer, error);
-
-    g_object_unref (writer);
-    return ret;
-}
-
-
-/************************************************************************/
-/* MooFileWriter
- */
-
-typedef struct MooLocalFileWriter MooLocalFileWriter;
-typedef struct MooLocalFileWriterClass MooLocalFileWriterClass;
-
-struct MooLocalFileWriter {
-    MooFileWriter base;
-
-    FILE *file;
-    char *filename;
-    char *temp_filename;
-    gboolean save_backup;
-    GError *error;
-};
-
-struct MooLocalFileWriterClass {
-    MooFileWriterClass base_class;
-};
-
-#define MOO_TYPE_LOCAL_FILE_WRITER (moo_local_file_writer_get_type ())
-
-MOO_DEFINE_TYPE_STATIC (MooLocalFileWriter, moo_local_file_writer, MOO_TYPE_FILE_WRITER)
-
-static gboolean
-same_content (const char *filename1,
-              const char *filename2)
-{
-    GMappedFile *file1, *file2;
-    char *content1, *content2;
-    gsize len;
-    gboolean equal = FALSE;
-
-    file1 = g_mapped_file_new (filename1, FALSE, NULL);
-    file2 = g_mapped_file_new (filename2, FALSE, NULL);
-
-    if (!file1 || !file2 ||
-        g_mapped_file_get_length (file1) != g_mapped_file_get_length (file2))
-            goto out;
-
-    len = g_mapped_file_get_length (file1);
-    content1 = g_mapped_file_get_contents (file1);
-    content2 = g_mapped_file_get_contents (file2);
-
-    if (memcmp (content1, content2, len) == 0)
-        equal = TRUE;
-
-out:
-    if (file1)
-        g_mapped_file_free (file1);
-    if (file2)
-        g_mapped_file_free (file2);
-    return equal;
-}
-
-static MooFileWriter *
-moo_local_file_writer_new (const char  *filename,
-                           gboolean     text_mode,
-                           gboolean     save_backup,
-                           GError     **error)
-{
-    int fd;
-    FILE *file;
-    char *temp_filename = NULL;
-    char *dirname = NULL;
-    MooLocalFileWriter *writer = NULL;
-
-    g_return_val_if_fail (filename != NULL, NULL);
-
-    dirname = g_path_get_dirname (filename);
-
-    if (_moo_mkdir_with_parents (dirname) != 0)
-    {
-        int err = errno;
-        char *display_name = g_filename_display_name (dirname);
-        g_set_error (error, G_FILE_ERROR,
-                     g_file_error_from_errno (err),
-                     "could not create directory '%s': %s",
-                     display_name, g_strerror (err));
-        g_free (display_name);
-        goto out;
-    }
-
-    temp_filename = g_strdup_printf ("%s" G_DIR_SEPARATOR_S ".cfg-tmp-XXXXXX", dirname);
-    errno = 0;
-    fd = g_mkstemp (temp_filename);
-
-    if (fd == -1)
-    {
-        int err = errno;
-        char *display_name = g_filename_display_name (temp_filename);
-        g_set_error (error, G_FILE_ERROR,
-                     g_file_error_from_errno (err),
-                     "could not create temporary file '%s': %s",
-                     display_name, g_strerror (err));
-        g_free (display_name);
-        goto out;
-    }
-
-    close (fd);
-
-    if (!(file = g_fopen (temp_filename, text_mode ? "w" : "wb")))
-    {
-        int err = errno;
-        char *display_name = g_filename_display_name (temp_filename);
-        g_set_error (error, G_FILE_ERROR,
-                     g_file_error_from_errno (err),
-                     "could not create temporary file '%s': %s",
-                     display_name, g_strerror (err));
-        g_free (display_name);
-        g_unlink (temp_filename);
-        goto out;
-    }
-
-    writer = g_object_new (MOO_TYPE_LOCAL_FILE_WRITER, NULL);
-    writer->file = file;
-    writer->temp_filename = temp_filename;
-    temp_filename = NULL;
-    writer->filename = g_strdup (filename);
-    writer->error = NULL;
-    writer->save_backup = save_backup;
-
-out:
-    g_free (temp_filename);
-    g_free (dirname);
-    return MOO_FILE_WRITER (writer);
-}
-
-MooFileWriter *
-moo_file_writer_new (const char  *filename,
-                     gboolean     save_backup,
-                     GError     **error)
-{
-    g_return_val_if_fail (filename != NULL, NULL);
-    g_return_val_if_fail (!error || !*error, NULL);
-    return moo_local_file_writer_new (filename, FALSE, save_backup, error);
-}
-
-MooFileWriter *
-moo_text_writer_new (const char  *filename,
-                     gboolean     save_backup,
-                     GError     **error)
-{
-    g_return_val_if_fail (filename != NULL, NULL);
-    g_return_val_if_fail (!error || !*error, NULL);
-    return moo_local_file_writer_new (filename, TRUE, save_backup, error);
-}
-
-
-static gboolean
-moo_local_file_writer_write (MooFileWriter *fwriter,
-                             const char    *data,
-                             gsize          len)
-{
-    MooLocalFileWriter *writer = (MooLocalFileWriter*) fwriter;
-
-    if (writer->error)
-        return FALSE;
-
-    while (len > 0)
-    {
-        gsize written;
-
-        errno = 0;
-        written = fwrite (data, 1, len, writer->file);
-
-        if (written < len && ferror (writer->file))
-        {
-            int err = errno;
-            g_set_error (&writer->error, G_FILE_ERROR,
-                         g_file_error_from_errno (err),
-                         "%s", g_strerror (err));
-            break;
-        }
-
-        len -= written;
-        data += written;
-    }
-
-    return writer->error == NULL;
-}
-
-G_GNUC_PRINTF (2, 0)
-static gboolean
-moo_local_file_writer_printf (MooFileWriter *fwriter,
-                              const char    *fmt,
-                              va_list        args)
-{
-    MooLocalFileWriter *writer = (MooLocalFileWriter*) fwriter;
-
-    if (writer->error)
-        return FALSE;
-
-    errno = 0;
-
-    if (vfprintf (writer->file, fmt, args) < 0)
-    {
-        int err = errno;
-        g_set_error (&writer->error, G_FILE_ERROR,
-                     g_file_error_from_errno (err),
-                     "%s", g_strerror (err));
-    }
-
-    return writer->error == NULL;
-}
-
-static gboolean
-moo_local_file_writer_close (MooFileWriter *fwriter,
-                             GError       **error)
-{
-    MooLocalFileWriter *writer = (MooLocalFileWriter*) fwriter;
-    gboolean retval;
-
-    if (!writer->error)
-    {
-        errno = 0;
-
-        if (fclose (writer->file) != 0)
-        {
-            int err = errno;
-            g_set_error (&writer->error, G_FILE_ERROR,
-                         g_file_error_from_errno (err),
-                         "%s", g_strerror (err));
-        }
-
-        writer->file = NULL;
-    }
-
-    if (!writer->error &&
-        !same_content (writer->filename, writer->temp_filename))
-    {
-        if (writer->save_backup)
-        {
-            char *bak_file;
-
-            bak_file = g_strdup_printf ("%s.bak", writer->filename);
-
-            if (g_file_test (writer->filename, G_FILE_TEST_EXISTS))
-            {
-                g_unlink (bak_file);
-                _moo_rename_file (writer->filename, bak_file, NULL);
-            }
-
-            g_free (bak_file);
-        }
-
-#ifdef __WIN32__
-        _moo_unlink (writer->filename);
-#endif
-        _moo_rename_file (writer->temp_filename,
-                          writer->filename,
-                          &writer->error);
-    }
-
-    g_unlink (writer->temp_filename);
-    g_free (writer->temp_filename);
-    g_free (writer->filename);
-    writer->temp_filename = NULL;
-    writer->filename = NULL;
-
-    retval = writer->error == NULL;
-
-    if (writer->error)
-        g_propagate_error (error, writer->error);
-
-    writer->error = NULL;
-    return retval;
-}
-
-
-static void
-moo_local_file_writer_class_init (MooLocalFileWriterClass *klass)
-{
-    MooFileWriterClass *writer_class = MOO_FILE_WRITER_CLASS (klass);
-    writer_class->meth_write = moo_local_file_writer_write;
-    writer_class->meth_printf = moo_local_file_writer_printf;
-    writer_class->meth_close = moo_local_file_writer_close;
-}
-
-static void
-moo_local_file_writer_init (MooLocalFileWriter *writer)
-{
-    writer->file = NULL;
-    writer->temp_filename = NULL;
-    writer->filename = NULL;
-    writer->error = NULL;
-}
-
-
-/************************************************************************/
-/* MooFileWriter
- */
-
-typedef struct MooStringWriter MooStringWriter;
-typedef struct MooStringWriterClass MooStringWriterClass;
-
-struct MooStringWriter {
-    MooFileWriter base;
-    GString *string;
-};
-
-struct MooStringWriterClass {
-    MooFileWriterClass base_class;
-};
-
-#define MOO_TYPE_STRING_WRITER (moo_string_writer_get_type ())
-
-MOO_DEFINE_TYPE_STATIC (MooStringWriter, moo_string_writer, MOO_TYPE_FILE_WRITER)
-
-static gboolean
-moo_string_writer_write (MooFileWriter *fwriter,
-                         const char    *data,
-                         gsize          len)
-{
-    MooStringWriter *writer = (MooStringWriter*) fwriter;
-    g_string_append_len (writer->string, data, len);
-    return TRUE;
-}
-
-G_GNUC_PRINTF (2, 0)
-static gboolean
-moo_string_writer_printf (MooFileWriter *fwriter,
-                          const char    *fmt,
-                          va_list        args)
-{
-    MooStringWriter *writer = (MooStringWriter*) fwriter;
-    g_string_append_vprintf (writer->string, fmt, args);
-    return TRUE;
-}
-
-static gboolean
-moo_string_writer_close (MooFileWriter *fwriter,
-                         G_GNUC_UNUSED GError **error)
-{
-    MooStringWriter *writer = (MooStringWriter*) fwriter;
-    g_string_free (writer->string, TRUE);
-    writer->string = NULL;
-    return TRUE;
-}
-
-static void
-moo_string_writer_class_init (MooStringWriterClass *klass)
-{
-    MooFileWriterClass *writer_class = MOO_FILE_WRITER_CLASS (klass);
-    writer_class->meth_write = moo_string_writer_write;
-    writer_class->meth_printf = moo_string_writer_printf;
-    writer_class->meth_close = moo_string_writer_close;
-}
-
-static void
-moo_string_writer_init (MooStringWriter *writer)
-{
-    writer->string = g_string_new (NULL);
-}
-
-MooFileWriter *
-moo_string_writer_new (void)
-{
-    return g_object_new (MOO_TYPE_STRING_WRITER, NULL);
-}
-
-const char *
-moo_string_writer_get_string (MooFileWriter *fwriter,
-                              gsize         *len)
-{
-    MooStringWriter *writer = (MooStringWriter*) fwriter;
-
-    g_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE ((writer), MOO_TYPE_STRING_WRITER), NULL);
-    g_return_val_if_fail (writer->string != NULL, NULL);
-
-    if (len)
-        *len = writer->string->len;
-
-    return writer->string->str;
-}
-
-
-#ifdef MOO_ENABLE_UNIT_TESTS
-
-#include <mooutils/mooutils-tests.h>
-
-static gboolean
-check_file_contents (const char *filename,
-                     const char *contents)
-{
-    gboolean equal;
-    char *real_contents;
-
-    if (!g_file_get_contents (filename, &real_contents, NULL, NULL))
-        return FALSE;
-
-    equal = strcmp (real_contents, contents) == 0;
-
-    g_free (real_contents);
-    return equal;
-}
-
-static void
-test_moo_file_writer (void)
-{
-    const char *dir;
-    char *my_dir, *filename, *bak_filename;
-    MooFileWriter *writer;
-    GError *error = NULL;
-
-    dir = moo_test_get_working_dir ();
-    my_dir = g_build_filename (dir, "cfg-writer", NULL);
-    filename = g_build_filename (my_dir, "configfile", NULL);
-    bak_filename = g_strdup_printf ("%s.bak", filename);
-
-    writer = moo_text_writer_new (filename, TRUE, &error);
-    TEST_ASSERT_MSG (writer != NULL,
-                     "moo_cfg_writer_new failed: %s",
-                     error ? error->message : "");
-    if (error)
-    {
-        g_error_free (error);
-        error = NULL;
-    }
-
-    if (writer)
-    {
-        moo_file_writer_write (writer, "first line\n", -1);
-        moo_file_writer_printf (writer, "second line #%d\n", 2);
-        moo_file_writer_write (writer, "third\nlalalala\n", 6);
-        TEST_ASSERT_MSG (moo_file_writer_close (writer, &error),
-                         "moo_file_writer_close failed: %s",
-                         error ? error->message : "");
-        if (error)
-        {
-            g_error_free (error);
-            error = NULL;
-        }
-
-#ifdef __WIN32__
-#define LE "\r\n"
-#else
-#define LE "\n"
-#endif
-
-        TEST_ASSERT (g_file_test (filename, G_FILE_TEST_EXISTS));
-        TEST_ASSERT (!g_file_test (bak_filename, G_FILE_TEST_EXISTS));
-        TEST_ASSERT (check_file_contents (filename, "first line" LE "second line #2" LE "third" LE));
-    }
-
-    writer = moo_text_writer_new (filename, TRUE, &error);
-    TEST_ASSERT_MSG (writer != NULL,
-                     "moo_text_writer_new failed: %s",
-                     error ? error->message : "");
-    if (writer)
-    {
-        moo_file_writer_write (writer, "First line\n", -1);
-        moo_file_writer_printf (writer, "Second line #%d\n", 2);
-        moo_file_writer_write (writer, "Third\nlalalala\n", 6);
-        TEST_ASSERT_MSG (moo_file_writer_close (writer, &error),
-                         "moo_file_writer_close failed: %s",
-                         error ? error->message : "");
-        if (error)
-        {
-            g_error_free (error);
-            error = NULL;
-        }
-
-        TEST_ASSERT (g_file_test (filename, G_FILE_TEST_EXISTS));
-        TEST_ASSERT (g_file_test (bak_filename, G_FILE_TEST_EXISTS));
-        TEST_ASSERT (check_file_contents (filename, "First line" LE "Second line #2" LE "Third" LE));
-        TEST_ASSERT (check_file_contents (bak_filename, "first line" LE "second line #2" LE "third" LE));
-        TEST_ASSERT (!same_content (bak_filename, filename));
-    }
-
-    TEST_ASSERT (_moo_remove_dir (my_dir, TRUE, NULL));
-
-#ifndef __WIN32__
-    writer = moo_text_writer_new ("/usr/test-mooutils-fs", TRUE, &error);
-#else
-    writer = moo_text_writer_new ("K:\\nowayyouhaveit\\file.ini", TRUE, &error);
-#endif
-    TEST_ASSERT (writer == NULL);
-    TEST_ASSERT (error != NULL);
-
-    if (error)
-        g_error_free (error);
-    error = NULL;
-
-    g_free (bak_filename);
-    g_free (filename);
-    g_free (my_dir);
-}
-
-void
-moo_test_moo_file_writer (void)
-{
-    MooTestSuite *suite;
-
-    suite = moo_test_suite_new ("MooFileWriter", NULL, NULL, NULL);
-
-    moo_test_suite_add_test (suite, "test of MooFileWriter",
-                             (MooTestFunc) test_moo_file_writer, NULL);
-}
-
-#endif /* MOO_ENABLE_UNIT_TESTS */
