@@ -1,22 +1,27 @@
 /*
  *   mooutils-misc.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-#include "moo-config-dirs.h"
+#include "config-dirs.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-file.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/mooi18n.h"
+#include "mooutils/mooatom.h"
+#include "mooutils/mooonce.h"
+#include "mooutils/mooutils-misc.h"
 #include "glade/moologwindow-gxml.h"
 #include <gtk/gtk.h>
 #include <glib/gmappedfile.h>
@@ -47,13 +52,13 @@
 
 
 G_LOCK_DEFINE_STATIC (moo_user_data_dir);
-G_LOCK_DEFINE_STATIC (moo_temp_dir);
 static char *moo_app_instance_name;
 static char *moo_user_data_dir;
 static char *moo_temp_dir;
 
-static void     moo_install_atexit      (void);
-static void     moo_remove_tempdir      (void);
+static void         moo_install_atexit      (void);
+static void         moo_remove_tempdir      (void);
+static const char  *moo_get_prgname         (void);
 
 
 #ifdef __WIN32__
@@ -745,11 +750,8 @@ moo_log_window_insert (MooLogWindow *log,
 static GLogFunc moo_log_func;
 static GPrintFunc moo_print_func;
 static GPrintFunc moo_printerr_func;
-static char *moo_log_file;
-static gboolean moo_log_file_written;
 static gboolean moo_log_handlers_set;
 
-
 static void
 set_print_funcs (GLogFunc   log_func,
                  GPrintFunc print_func,
@@ -778,9 +780,7 @@ set_print_funcs (GLogFunc   log_func,
     set (NULL);
 #undef set
 
-#if GLIB_CHECK_VERSION(2,6,0)
     g_log_set_default_handler (log_func, NULL);
-#endif /* GLIB_CHECK_VERSION(2,6,0) */
 }
 
 
@@ -813,6 +813,23 @@ moo_reset_log_func (void)
  * Display log messages in a window
  */
 
+#if !defined(__WIN32__) || !defined(DEBUG)
+
+static void
+on_warning_or_critical_message (G_GNUC_UNUSED const char *message)
+{
+}
+
+#else /* __WIN32__ && DEBUG */
+
+static void
+on_warning_or_critical_message (const char *message)
+{
+    _moo_abort_debug_ignore (MOO_CODE_LOC_UNKNOWN, message);
+}
+
+#endif
+
 static void
 log_func_window (const gchar    *log_domain,
                  GLogLevelFlags  flags,
@@ -820,8 +837,6 @@ log_func_window (const gchar    *log_domain,
                  G_GNUC_UNUSED gpointer dummy)
 {
     char *text;
-    GtkTextTag *tag;
-    MooLogWindow *log;
 
 #ifdef __WIN32__
     if (flags & (G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION))
@@ -831,21 +846,40 @@ log_func_window (const gchar    *log_domain,
     }
 #endif /* __WIN32__ */
 
+    if (!g_utf8_validate (message, -1, NULL))
+        message = "<corrupted string, invalid UTF8>";
+
     if (log_domain)
         text = g_strdup_printf ("%s: %s\n", log_domain, message);
     else
         text = g_strdup_printf ("%s\n", message);
 
-    log = moo_log_window ();
+    if (flags < G_LOG_LEVEL_MESSAGE)
+        on_warning_or_critical_message (text);
 
-    if (flags >= G_LOG_LEVEL_MESSAGE)
-        tag = log->message_tag;
-    else if (flags >= G_LOG_LEVEL_WARNING)
-        tag = log->warning_tag;
-    else
-        tag = log->critical_tag;
+    {
+        GtkTextTag *tag;
+        MooLogWindow *log;
+
+        gdk_threads_enter ();
+
+        if ((log = moo_log_window ()))
+        {
+            if (flags >= G_LOG_LEVEL_MESSAGE)
+                tag = log->message_tag;
+            else if (flags >= G_LOG_LEVEL_WARNING)
+                tag = log->warning_tag;
+            else
+                tag = log->critical_tag;
+
+            moo_log_window_insert (log, text, tag);
+            if (flags <= G_LOG_LEVEL_WARNING)
+                gtk_window_present (GTK_WINDOW (log->window));
+        }
+
+        gdk_threads_leave ();
+    }
 
-    moo_log_window_insert (log, text, tag);
     g_free (text);
 }
 
@@ -853,15 +887,21 @@ log_func_window (const gchar    *log_domain,
 static void
 print_func_window (const char *string)
 {
-    MooLogWindow *log = moo_log_window ();
-    moo_log_window_insert (log, string, NULL);
+    MooLogWindow *log;
+    gdk_threads_enter ();
+    if ((log = moo_log_window ()))
+        moo_log_window_insert (log, string, NULL);
+    gdk_threads_leave ();
 }
 
 static void
 printerr_func_window (const char *string)
 {
-    MooLogWindow *log = moo_log_window ();
-    moo_log_window_insert (log, string, log->warning_tag);
+    MooLogWindow *log;
+    gdk_threads_enter ();
+    if ((log = moo_log_window ()))
+        moo_log_window_insert (log, string, log->warning_tag);
+    gdk_threads_leave ();
 }
 
 
@@ -881,12 +921,22 @@ moo_set_log_func_window (gboolean show_now)
  * Write to a file
  */
 
+static char *moo_log_file;
+static gboolean moo_log_file_written;
+static GStaticRecMutex moo_log_file_mutex = G_STATIC_REC_MUTEX_INIT;
+
 static void
 print_func_file (const char *string)
 {
     FILE *file;
 
-    g_return_if_fail (moo_log_file != NULL);
+    g_static_rec_mutex_lock (&moo_log_file_mutex);
+
+    if (!moo_log_file)
+    {
+        g_static_rec_mutex_unlock (&moo_log_file_mutex);
+        g_return_if_reached ();
+    }
 
     if (!moo_log_file_written)
     {
@@ -907,6 +957,8 @@ print_func_file (const char *string)
     {
         /* TODO ??? */
     }
+
+    g_static_rec_mutex_unlock (&moo_log_file_mutex);
 }
 
 
@@ -939,9 +991,11 @@ log_func_file (const char       *log_domain,
 void
 moo_set_log_func_file (const char *log_file)
 {
+    g_static_rec_mutex_lock (&moo_log_file_mutex);
     g_free (moo_log_file);
     moo_log_file = g_strdup (log_file);
     set_print_funcs (log_func_file, print_func_file, print_func_file);
+    g_static_rec_mutex_unlock (&moo_log_file_mutex);
 }
 
 
@@ -982,11 +1036,18 @@ moo_set_log_func_silent (void)
 /* Very useful function
  */
 
-void
+void MOO_NORETURN
 moo_segfault (void)
 {
     char *var = (char*) -1;
     var[18] = 8;
+    moo_abort ();
+}
+
+void MOO_NORETURN
+moo_abort (void)
+{
+    abort ();
 }
 
 
@@ -1120,21 +1181,17 @@ _moo_menu_item_set_label (GtkWidget  *item,
 const char *
 _moo_get_pid_string (void)
 {
-    G_LOCK_DEFINE_STATIC (moo_pid_string);
     static char *moo_pid_string;
 
-    G_LOCK (moo_pid_string);
+    MOO_DO_ONCE_BEGIN
 
-    if (!moo_pid_string)
-    {
 #ifdef __WIN32__
-        moo_pid_string = g_strdup_printf ("%ld", GetCurrentProcessId ());
+    moo_pid_string = g_strdup_printf ("%ld", GetCurrentProcessId ());
 #else
-        moo_pid_string = g_strdup_printf ("%ld", (long) getpid ());
+    moo_pid_string = g_strdup_printf ("%ld", (long) getpid ());
 #endif
-    }
 
-    G_UNLOCK (moo_pid_string);
+    MOO_DO_ONCE_END
 
     return moo_pid_string;
 }
@@ -1150,7 +1207,7 @@ moo_get_prgname (void)
     if (!name)
     {
         g_critical ("%s: program name not set", G_STRLOC);
-        name = "ggap";
+        name = "medit";
     }
 
     return name;
@@ -1165,9 +1222,7 @@ moo_tempnam (void)
     static int counter;
     G_LOCK_DEFINE_STATIC (counter);
 
-    G_LOCK (moo_temp_dir);
-
-    if (!moo_temp_dir)
+    MOO_DO_ONCE_BEGIN
     {
         char *dirname = NULL;
         const char *short_name;
@@ -1192,8 +1247,7 @@ moo_tempnam (void)
         moo_temp_dir = dirname;
         moo_install_atexit ();
     }
-
-    G_UNLOCK (moo_temp_dir);
+    MOO_DO_ONCE_END
 
     g_return_val_if_fail (moo_temp_dir != NULL, NULL);
 
@@ -1269,7 +1323,7 @@ moo_cleanup (void)
 }
 
 
-static char *
+char *
 moo_get_user_cache_dir (void)
 {
     return g_build_filename (g_get_user_cache_dir (), moo_get_prgname (), NULL);
@@ -1299,7 +1353,7 @@ moo_get_user_data_dir (void)
 }
 
 void
-_moo_set_user_data_dir (const char *path)
+moo_set_user_data_dir (const char *path)
 {
     G_LOCK (moo_user_data_dir);
 
@@ -1387,9 +1441,10 @@ add_dir_list_from_env (GPtrArray  *list,
 }
 
 
-char **
-moo_get_data_dirs (MooDataDirType type,
-                   guint         *n_dirs)
+static char **
+moo_get_data_dirs_real (MooDataDirType   type,
+                        char           **add,
+                        guint           *n_dirs)
 {
     static char **moo_data_dirs[2];
     static guint n_data_dirs[2];
@@ -1399,6 +1454,9 @@ moo_get_data_dirs (MooDataDirType type,
 
     G_LOCK (moo_data_dirs);
 
+    if (add && moo_data_dirs[type])
+        g_critical ("moo_add_data_dirs() called too late, ignored");
+
     if (!moo_data_dirs[type])
     {
         const char *env[2];
@@ -1415,6 +1473,9 @@ moo_get_data_dirs (MooDataDirType type,
 
         g_ptr_array_add (all_dirs, moo_get_user_data_dir ());
 
+        while (add && *add)
+            g_ptr_array_add (all_dirs, g_strdup (*add++));
+
         /* environment variables override everything */
         if (env[0] || env[1])
         {
@@ -1488,6 +1549,19 @@ moo_get_data_dirs (MooDataDirType type,
     return g_strdupv (moo_data_dirs[type]);
 }
 
+char **
+moo_get_data_dirs (MooDataDirType type,
+                   guint         *n_dirs)
+{
+    return moo_get_data_dirs_real (type, NULL, n_dirs);
+}
+
+void
+moo_add_data_dirs (char **dirs)
+{
+    g_strfreev (moo_get_data_dirs_real (MOO_DATA_SHARE, dirs, NULL));
+}
+
 
 const char *
 moo_get_locale_dir (void)
@@ -1501,56 +1575,6 @@ moo_get_locale_dir (void)
 }
 
 
-/* For xdgmime: should not contain duplicates */
-const char* const *
-_moo_get_shared_data_dirs (void)
-{
-    static char **data_dirs = NULL;
-    G_LOCK_DEFINE_STATIC(data_dirs);
-
-    G_LOCK (data_dirs);
-
-    if (!data_dirs)
-    {
-        guint i, n_dirs;
-        GSList *dirs = NULL;
-        const char* const *sys_dirs;
-
-        sys_dirs = g_get_system_data_dirs ();
-
-#ifndef __WIN32__
-        dirs = g_slist_prepend (NULL, _moo_normalize_file_path (g_get_user_data_dir ()));
-#endif
-
-        for (i = 0; sys_dirs[i] && sys_dirs[i][0]; ++i)
-        {
-            char *norm = _moo_normalize_file_path (sys_dirs[i]);
-
-            if (norm && !g_slist_find_custom (dirs, norm, (GCompareFunc) strcmp))
-                dirs = g_slist_prepend (dirs, norm);
-            else
-                g_free (norm);
-        }
-
-        dirs = g_slist_reverse (dirs);
-        n_dirs = g_slist_length (dirs);
-        data_dirs = g_new (char*, n_dirs + 1);
-
-        for (i = 0; i < n_dirs; ++i)
-        {
-            data_dirs[i] = dirs->data;
-            dirs = g_slist_delete_link (dirs, dirs);
-        }
-
-        data_dirs[n_dirs] = NULL;
-    }
-
-    G_UNLOCK (data_dirs);
-
-    return (const char* const*) data_dirs;
-}
-
-
 char **
 moo_get_data_subdirs (const char    *subdir,
                       MooDataDirType type,
@@ -1645,7 +1669,7 @@ save_config_file (const char     *dir,
     g_return_val_if_fail (filename != NULL, FALSE);
     g_return_val_if_fail (content != NULL, FALSE);
 
-    if (!(writer = moo_text_writer_new (filename, TRUE, error)))
+    if (!(writer = moo_config_writer_new (filename, TRUE, error)))
         return FALSE;
 
     moo_file_writer_write (writer, content, len);
@@ -1751,6 +1775,177 @@ _moo_widget_set_tooltip (GtkWidget  *widget,
 }
 
 
+/*******************************************************************************
+ * Asserts
+ */
+
+#if !defined(__WIN32__)
+
+void
+_moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
+{
+    if (moo_code_loc_valid (loc))
+        g_printerr ("In file %s, line %d, function %s:\n%s\n", loc.file, loc.line, loc.func, message);
+    else
+        g_printerr ("%s\n", message);
+    g_on_error_query (moo_get_prgname());
+}
+
+#else /* !__WIN32__ */
+
+static void
+break_into_debugger (void)
+{
+    RaiseException(0, EXCEPTION_NONCONTINUABLE, 0, NULL);
+}
+
+void
+_moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
+{
+    gboolean skip = FALSE;
+    static GHashTable *locs_hash;
+    char *loc_id = NULL;
+    gboolean use_location;
+
+    use_location = moo_code_loc_valid (loc);
+
+    if (use_location)
+        loc_id = g_strdup_printf ("%s#%d#%d", loc.file, loc.line, loc.counter);
+    else
+        loc_id = g_strdup (message);
+
+    if (loc_id != NULL)
+    {
+        if (!locs_hash)
+            locs_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+        if (g_hash_table_lookup (locs_hash, loc_id))
+            skip = TRUE;
+    }
+
+    if (!skip)
+    {
+        int ret;
+
+        if (use_location)
+            ret = _moo_win32_message_box (NULL, MB_ABORTRETRYIGNORE, NULL,
+                                          "In file %s, line %d, function %s:\n%s",
+                                          loc.file, loc.line, loc.func, message);
+        else
+            ret = _moo_win32_message_box (NULL, MB_ABORTRETRYIGNORE, NULL,
+                                          "%s", message);
+
+        switch (ret)
+        {
+            case IDABORT:
+                TerminateProcess (GetCurrentProcess (), 1);
+                break;
+
+            case IDRETRY:
+                break_into_debugger ();
+                break;
+
+            case IDIGNORE:
+                if (GetKeyState(VK_CONTROL) < 0)
+                    skip = TRUE;
+                break;
+
+            default:
+                moo_abort ();
+                break;
+        }
+
+        if (skip && loc_id)
+        {
+            g_hash_table_insert (locs_hash, loc_id, GINT_TO_POINTER (1));
+            loc_id = NULL;
+        }
+    }
+
+    g_free (loc_id);
+}
+
+#endif /* !__WIN32__ */
+
+#ifndef MOO_DEV_MODE
+NORETURN
+#endif
+void
+_moo_assert_message (MooCodeLoc loc, const char *message)
+{
+#if defined(MOO_DEV_MODE) && !defined(__WIN32__)
+    _moo_abort_debug_ignore (loc, message);
+#else
+    g_error ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
+#endif
+}
+
+void
+_moo_log (MooCodeLoc loc, GLogLevelFlags flags, const char *format, ...)
+{
+    va_list args;
+    va_start (args, format);
+    _moo_logv (loc, flags, format, args);
+    va_end (args);
+}
+
+static gboolean
+moo_log_debug_enabled (void)
+{
+    static int enabled = -1;
+
+    if (enabled == -1)
+    {
+        enabled = moo_debug_enabled ("misc",
+#ifdef MOO_DEBUG
+                                     TRUE);
+#else
+                                     FALSE);
+#endif
+    }
+
+    return enabled;
+}
+
+void _moo_logv (MooCodeLoc loc, GLogLevelFlags flags, const char *format, va_list args)
+{
+    char *message = g_strdup_vprintf (format, args);
+
+    if (flags >= G_LOG_LEVEL_DEBUG && !moo_log_debug_enabled ())
+        return;
+
+#if defined(MOO_DEV_MODE) && !defined(__WIN32__)
+    if (flags < G_LOG_LEVEL_MESSAGE)
+    {
+        _moo_abort_debug_ignore (loc, message);
+        flags = G_LOG_LEVEL_MESSAGE;
+    }
+#endif
+
+    if (moo_code_loc_valid (loc))
+        g_log (G_LOG_DOMAIN, flags,
+               "In file %s, line %d, function %s: %s",
+               loc.file, loc.line, loc.func, message);
+    else
+        g_log (G_LOG_DOMAIN, flags, "%s", message);
+
+    g_free (message);
+}
+
+void MOO_NORETURN _moo_error (MooCodeLoc loc, const char *format, ...)
+{
+    va_list args;
+    va_start (args, format);
+    _moo_errorv (loc, format, args);
+    va_end (args);
+}
+
+void MOO_NORETURN _moo_errorv (MooCodeLoc loc, const char *format, va_list args)
+{
+    MOO_UNUSED (loc);
+    g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, format, args);
+    moo_abort ();
+}
+
 
 /*******************************************************************************
  * Former eggregex stuff
@@ -1946,6 +2141,55 @@ moo_line_reader_get_line (MooLineReader *lr,
 }
 
 
+gboolean
+moo_find_line_end (const char *string,
+                   gssize      len,
+                   gsize      *line_len,
+                   gsize      *lt_len)
+{
+    const char *le, *end;
+    gsize le_len;
+
+    g_return_val_if_fail (string != NULL || len == 0, FALSE);
+
+    if (len < 0)
+        len = strlen (string);
+
+    for (le = string, le_len = 0, end = string + len; le < end && le_len == 0; )
+    {
+        switch (*le)
+        {
+            case '\r':
+                if (le + 1 < end && le[1] == '\n')
+                    le_len = 2;
+                else
+                    le_len = 1;
+                break;
+            case '\n':
+                le_len = 1;
+                break;
+            case '\xe2': /* Unicode paragraph separator "\xe2\x80\xa9" */
+                if (le + 2 < end && le[1] == '\x80' && le[2] == '\xa9')
+                {
+                    le_len = 3;
+                    break;
+                }
+                /* fallthrough */
+            default:
+                le += 1;
+                break;
+        }
+    }
+
+    if (line_len != NULL)
+        *line_len = le - string;
+    if (lt_len != NULL)
+        *lt_len = le_len;
+
+    return le_len != 0;
+}
+
+
 char **
 _moo_strv_reverse (char **str_array)
 {
@@ -1967,14 +2211,22 @@ _moo_strv_reverse (char **str_array)
 }
 
 
-#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
+gboolean
+_moo_str_equal (const char *s1,
+                const char *s2)
+{
+    return strcmp (s1 ? s1 : "", s2 ? s2 : "") == 0;
+}
+
+
+#if defined(__WIN32__) && !defined(MOO_DEBUG)
 static guint saved_win32_error_mode;
 #endif
 
 void
 moo_disable_win32_error_message (void)
 {
-#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
+#if defined(__WIN32__) && !defined(MOO_DEBUG)
     saved_win32_error_mode = SetErrorMode (SEM_NOOPENFILEERRORBOX | SEM_FAILCRITICALERRORS);
 #endif
 }
@@ -1982,7 +2234,7 @@ moo_disable_win32_error_message (void)
 void
 moo_enable_win32_error_message (void)
 {
-#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
+#if defined(__WIN32__) && !defined(MOO_DEBUG)
     SetErrorMode (saved_win32_error_mode);
 #endif
 }
@@ -2167,8 +2419,16 @@ _moo_intern_string (const char *string)
 #endif
 }
 
+GdkAtom
+moo_atom_uri_list (void)
+{
+    MOO_DEFINE_ATOM_ (text/uri-list)
+}
+
 
 static char *debug_domains;
+void _moo_set_debug (const char *domains);
+gboolean moo_debug_enabled (const char *domain, gboolean def_enabled);
 
 void
 _moo_set_debug (const char *domains)
@@ -2210,33 +2470,6 @@ moo_debug_enabled (const char *domain,
 }
 
 
-#undef _moo_message
-void G_GNUC_PRINTF(1,2)
-_moo_message (const char *format,
-              ...)
-{
-    static int enabled = -1;
-
-    if (enabled == -1)
-    {
-        enabled = moo_debug_enabled ("misc",
-#ifdef MOO_DEBUG_ENABLED
-                                     TRUE);
-#else
-                                     FALSE);
-#endif
-    }
-
-    if (enabled)
-    {
-        va_list args;
-        va_start (args, format);
-        g_logv (G_LOG_DOMAIN "-debug", G_LOG_LEVEL_MESSAGE, format, args);
-        va_end (args);
-    }
-}
-
-
 #ifdef MOO_ENABLE_UNIT_TESTS
 
 #include <mooutils/mooutils-tests.h>
